#include "registry.hpp"

Registry::Registry(const std::string& subkey, const HKEY mainkey, bool create)
{
  if(subkey.empty())
    throw RegistryError();

  if(mainkey==HKEY_DYN_DATA)
    create=false;

  DWORD e;
  if(create)
    e=RegCreateKeyEx(
      mainkey,
      subkey.c_str(), 
      0, 
      NULL, 
      REG_OPTION_NON_VOLATILE, 
      KEY_ALL_ACCESS, 
      NULL, 
      &hKey, 
      NULL);
  else
    e=RegOpenKeyEx(mainkey, subkey.c_str(), 0, KEY_READ, &hKey);

  if(e!=ERROR_SUCCESS)
  {
    hKey=0;
    throw RegistryError();
  }
}

Registry::~Registry()
{
  if(hKey)
    RegCloseKey(hKey);
}

void Registry::set(const std::string& key, const std::string& val)
{
  DWORD e=RegSetValueEx(
    hKey, 
    key.c_str(),
    0,
    REG_SZ, 
    reinterpret_cast<const BYTE*>(val.c_str()),
    val.size());
  if(e!=ERROR_SUCCESS)
    throw RegistryError();
}

void Registry::set(const std::string& key, const DWORD i)
{
  DWORD e=RegSetValueEx(
    hKey, 
    key.c_str(),
    0,
    REG_DWORD, 
    reinterpret_cast<const BYTE*>(&i),
    sizeof(i));
  if(e!=ERROR_SUCCESS)
    throw RegistryError();
}

std::string Registry::getString(const std::string& key)
{
  char buf[1024];
  DWORD bufSize=sizeof(buf);
  DWORD type;
  DWORD e=RegQueryValueEx(
    hKey,
    key.c_str(),
    0,
    &type,
    reinterpret_cast<BYTE*>(buf),
    &bufSize
  );
  if(e!=ERROR_SUCCESS)
    throw RegistryError();
  if(type!=REG_SZ)
    throw RegistryError();
  if(bufSize>=sizeof(buf)-1)
    buf[sizeof(buf)-1]=0;
  return buf;
}

DWORD Registry::getInt(const std::string& key)
{
  BYTE buf[1024];
  DWORD bufSize=sizeof(buf);
  DWORD type;
  DWORD e=RegQueryValueEx(
    hKey,
    key.c_str(),
    0,
    &type,
    buf,
    &bufSize
  );
  if(e!=ERROR_SUCCESS)
    throw RegistryError();
  if(type!=REG_DWORD)
    throw RegistryError();
  return reinterpret_cast<DWORD*>(buf)[0];
}

DWORD Registry::get(const std::string& key, BYTE *buf, DWORD& bufSize)
{
  DWORD type;
  DWORD e=RegQueryValueEx(
    hKey,
    key.c_str(),
    0,
    &type,
    buf,
    &bufSize
  );
  if(e!=ERROR_SUCCESS)
    throw RegistryError();
  return type;
}

void Registry::del(const std::string& key)
{
  if(RegDeleteKey(hKey, key.c_str()) != ERROR_SUCCESS)
    throw RegistryError();
}
